package aop.example;

import aop.example.model.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;

/**
 *  
 *      
 *      :)
 * @author Zubairov Renat
 */
public aspect AuthenticationAspect {
	
	// pointcut       
	//    
	//      
	public pointcut doFilterMethod(ServletRequest srequest, ServletResponse sresponse, EntranceFilter filter) :
		execution(void aop.example.EntranceFilter.doFilter(ServletRequest, ServletResponse, FilterChain))
			&& args(srequest, sresponse, FilterChain) && this(filter);


	// pointcut      
	//       ViewServlet doGet
	//       HttpServletRequest
	public pointcut authenticationNeeded(HttpServletRequest request) :
		execution(* aop.example.ViewServlet.doGet(HttpServletRequest, HttpServletResponse)) && args(request, HttpServletResponse);

	/**
	 * Advice      
	 *       ,   
	 */
	before(HttpServletRequest request) : authenticationNeeded(request) {
		//          
		//  ,    
		if (request.getParameterMap().size() > 0) {
			if (request.getSession().getAttribute(EntranceFilter.USER_KEY) == null) {
				throw new AuthenticationException("User not logged in");
			}
			if (!(request.getSession().getAttribute(EntranceFilter.USER_KEY) instanceof User)) {
				throw new AuthenticationException("Anonymous user access denided");
			}
		}
	}

	/**
	 * Around advice ( pointcut -   doFilter) 
	 *      
	 *       
	 */
	void around(ServletRequest srequest, ServletResponse sresponse, EntranceFilter filter) throws IOException, ServletException : doFilterMethod(srequest, sresponse, filter) {
		try {
			//  
			proceed(srequest, sresponse, filter);
		} catch (AuthenticationException e) {
			//      
			srequest.setAttribute("error_message", e.getMessage());
			//     
			filter.getConfig().getServletContext().getRequestDispatcher("login.vm").forward(srequest, sresponse);
		}
	}
	
}
