package aop.example;

import aop.example.model.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;

/**
 *  
 * @author Zubairov Renat
 */
public aspect AuthorizationAspect {
	
	/**
	 * pointcut      ServletRequest 
	 *           
	 *   pointcut ( "")  
	 */
	pointcut requestMethod(ServletRequest request) :
		execution(* aop.example.EntranceFilter.doFilter(ServletRequest, ServletResponse, FilterChain)) 
		&& args(request, ServletResponse, FilterChain);

	/**
	 * pointcut        
	 * 
	 */
	public pointcut doFilterMethod(ServletRequest srequest, ServletResponse sresponse, EntranceFilter filter) :
		execution(void aop.example.EntranceFilter.doFilter(ServletRequest, ServletResponse, FilterChain))
			&& args(srequest, sresponse, FilterChain) && this(filter);
	
	
	//       
	pointcut readMethods(Object object) : execution (public * aop.example.model.*.get*(..)) && this(object);

	//      
	pointcut addMethods(Object object) : execution (public * aop.example.model.*.add*(..)) && this(object);
	
	//      
	pointcut deleteMethods(Album album) : execution (public * aop.example.model.AlbumList.deleteAlbum(Album)) && args(album);
	
	//      (-)
	pointcut controlledRead(Object object) : execution(public boolean aop.example.model.Controlled+.isReadable()) && this(object);
	
	//      (-)
	pointcut controlledAdd(Object object) : execution(public boolean aop.example.model.Controlled+.isAddable()) && this(object);

	//      (-)
	pointcut controlledDelete(Object object) : execution(public boolean aop.example.model.Controlled+.isDeletable()) && this(object);
	
	//       
	//     
	//    pointcut -   "" 
	pointcut readAccess(ServletRequest request, Object object) :
		cflow(requestMethod(request)) && readMethods(object);

	//      
	pointcut addAccess(ServletRequest request, Object object) :
		cflow(requestMethod(request)) && addMethods(object);

	//      
	pointcut deleteAccess(ServletRequest request, Album album) :
		cflow(requestMethod(request)) && deleteMethods(album);
		
	// -  
	pointcut readCheck(ServletRequest request, Object object) :
		cflow(requestMethod(request)) && controlledRead(object);

	// -  
	pointcut addCheck(ServletRequest request, Object object) :
		cflow(requestMethod(request)) && controlledAdd(object);

	// -  
	pointcut deleteCheck(ServletRequest request, Object object) :
		cflow(requestMethod(request)) && controlledDelete(object);

	/**
	 * Around advice  
	 *       
	 */
	void around(ServletRequest srequest, ServletResponse sresponse, EntranceFilter filter) throws IOException, ServletException : doFilterMethod(srequest, sresponse, filter) {
		try {
			//   
			proceed(srequest, sresponse, filter);
		} catch (AuthorizationException e) {
			//  
			srequest.setAttribute("error_message", e.getMessage());
			//       
			filter.getConfig().getServletContext().getRequestDispatcher("error.vm").forward(srequest, sresponse);
		}
	}

		
	/**
	 * Before advice   
	 */
	before(ServletRequest request, Object object) : readAccess(request, object) {
		if (!AuthHelper.isAbleToRead(extractUser(request), object)) {
			throw new AuthorizationException("Read access not allowed");
		}
	}
	
	/**
	 * Before advice   
	 */
	before(ServletRequest request, Object object) : addAccess(request, object) {
		if (!AuthHelper.isAbleToAdd(extractUser(request), object)) {
			throw new AuthorizationException("Add access not allowed");
		}
	}

	/**
	 * Before advice   
	 */
	before(ServletRequest request, Album album) : deleteAccess(request, album) {
		if (!AuthHelper.isAbleToDelete(extractUser(request), album)) {
			throw new AuthorizationException("Delete access not allowed");
		}
	}
	
	/**
	 * Around advice -,     
	 * ,        
	 * 
	 *     - ..    
	 * ,      
	 */
	boolean around(ServletRequest request, Object object) : deleteCheck(request, object) {
		return AuthHelper.isAbleToDelete(extractUser(request), object); 
	}
	
	/**
	 *       
	 */
	private AnonymousUser extractUser(ServletRequest request) {
		return (AnonymousUser)((HttpServletRequest) request).getSession().getAttribute(EntranceFilter.USER_KEY);
	}

}
