package aop.example;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.LogRecord;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import aop.example.model.AlbumList;
import aop.example.model.AnonymousUser;
import aop.example.model.Album;
import aop.example.model.User;

/**
 *      , ..   
 *    .
 *     ,       
 *        AnonymousUser
 *   AlbumList     
 * @author Zubairov Renat
 */
public class EntranceFilter implements javax.servlet.Filter {
	
	/**
	 *      
	 *    
	 */
	public static final String USER_KEY = "CURRENT_USER";
	
	/**
	 *  ,     ServletContext
	 *    RequestDispatcher
	 */
	private FilterConfig config = null;

	/**
	 * Empty constructor 
	 */
	public EntranceFilter() {
		super();
	}

	/**
	 * @see java.util.logging.Filter#isLoggable(java.util.logging.LogRecord)
	 */
	public boolean isLoggable(LogRecord record) {
		return false;
	}
	
	/**
	 *  
	 * @see javax.servlet.Filter.doFilter(..)
	 */
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) 
				   throws IOException, ServletException
	{
		if (request instanceof HttpServletRequest) {
			//  
			HttpSession session = ((HttpServletRequest) request).getSession();
			if (session.getAttribute(USER_KEY) == null) {
				//    ,     
				session.setAttribute(USER_KEY, new AnonymousUser());
			}
			if (session.getAttribute("albums") == null) {
				//      ,   
				//      
				//  AlbumList
				session.setAttribute("albums", new AlbumList());
			}
		}
		//   
		chain.doFilter(request, response);
	}
	
	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		config = null;
	}

	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig aconfig) throws ServletException {
		config = aconfig;
	}

	

	/**
	 *       
	 *       
	 * @return
	 */
	public FilterConfig getConfig() {
		return config;
	}

}
